//**************************************************************************
// ﾌｧｲﾙ内容     I2C液晶 AQM0802 制御ライブラリ
// バージョン   Ver.2.00
// Date         2024.08.11
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "mcr_aqm0802_soft_i2c.h"

//#define i2c_SDA_0();    digitalWrite( _sda, 0 );pinMode( _sda, OUTPUT ); // SDA = "0"
#define i2c_SDA_0();    pinMode( _sda, OUTPUT );                         // SDA = "0" OUTPUTにするだけで"0"出力になる？
#define i2c_SDA_1();    pinMode( _sda, INPUT );                          // SDA = "1"
//#define i2c_SCL_0();    digitalWrite( _scl, 0 );pinMode( _scl, OUTPUT ); // SCL = "0"
#define i2c_SCL_0();    pinMode( _scl, OUTPUT );                         // SCL = "0" OUTPUTにするだけで"0"出力になる？
#define i2c_SCL_1();    pinMode( _scl, INPUT );                          // SCL = "1"

//**********************************************************************
// ソフトウェア beginTransmission
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::soft_i2c_write( unsigned char data ) {
  for( int i = 0; i<8; i++ ) {
    // data set
    if( data & 0x80 ) {
        i2c_SDA_1();
    } else {
        i2c_SDA_0();
    }

    // SCL ON
    i2c_SCL_1();

    // SCL OFF
    i2c_SCL_0();

    // 次の準備
    data <<= 1;
  }

  i2c_SDA_1();

  // SCL ON
  i2c_SCL_1();

  // SCL OFF
  i2c_SCL_0();
}

//**********************************************************************
// ソフトウェア beginTransmission
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::soft_i2c_beginTransmission( unsigned char data ) {
  i2c_SDA_1();
  i2c_SCL_1();

  i2c_SDA_0();
  i2c_SCL_1();

  i2c_SDA_0();
  i2c_SCL_0();

  soft_i2c_write( data << 1 );
}

//**********************************************************************
// ソフトウェア endTransmission
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::soft_i2c_endTransmission( void ) {
  i2c_SDA_0();
  i2c_SCL_0();

  i2c_SDA_0();
  i2c_SCL_1();

  i2c_SDA_1();
  i2c_SCL_1();
}

//**********************************************************************
// AQM0802へコマンド書き込み
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::writeCmd(byte command) {
  soft_i2c_beginTransmission( 0x3e );
  soft_i2c_write( 0x00 );
  soft_i2c_write(command);
  soft_i2c_endTransmission();
  delayMicroseconds( 30 );
}

//**********************************************************************
// AQM0802へ表示データ書き込み
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::writeData(byte data) {
  soft_i2c_beginTransmission( 0x3e );
  soft_i2c_write( 0x40 );
  soft_i2c_write( data );
  soft_i2c_endTransmission();
  delayMicroseconds( 100 );
}

//**********************************************************************
// コントラストセット 0～63 5Vのときは11～12がお勧めです
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::setContrast( byte c ) {
  if( c <  1 ) c =  1;
  if( c > 63 ) c = 63;
  contrast = c;

  writeCmd( 0x39 );                             // Function set
  writeCmd( 0x70 | ( contrast & 0x0f) );        // Contrast set(bit3,2,1,0)
  writeCmd( 0x54 | ( (contrast>>4) & 0x03) );   // Power/ICON/Contrast set(bit5,4)
  writeCmd( 0x38 );                             // Function set
}

//**********************************************************************
// 液晶の文字位置セット x=0がいちばん左、y=0がいちばん上
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::setPosition( char x, char y ) {
  byte yy = 0x00;

  if( x >= LCD_MAX_X ) return;
  if( y >= LCD_MAX_Y ) return;

  lcdPosition_x = x;
  lcdPosition_y = y;

  if( y == 1 ) {
    yy = 0x40;
  } else if( y == 2 ) {
    yy = 0x14;
  } else if( y == 3 ) {
    yy = 0x54;
  }
  writeCmd( 0x80 + x + yy );
}

//**********************************************************************
// AQM0802用のprintf関数
//**********************************************************************
int MCR_AQM0802_SOFT_I2C::printf( const char *format, ... ) {
  va_list argptr;
  char    *p;
  int     ret = 0;
  char    work[32];   // 一時作業エリア

  va_start(argptr, format);
  ret = vsprintf( work, format, argptr );
  va_end(argptr);

  if( ret > 0 ) {
    // vsprintfが正常なら液晶バッファへ転送
    p = work;
    while( *p ) {
      writeData( *p++ );
      lcdPosition_x++;
      if( lcdPosition_x >= LCD_MAX_X ) {
        lcdPosition_x = 0;
        lcdPosition_y++;
        if( lcdPosition_y >= LCD_MAX_Y ) {
          lcdPosition_y = 0;
        }
        setPosition( 0 , lcdPosition_y );
      }
    }
  }
  return ret;
}

//**********************************************************************
// AQM0802　初期化とコントラストセット
//**********************************************************************
void MCR_AQM0802_SOFT_I2C::init( uint16_t sda_pin, uint16_t scl_pin, uint8_t c ) {
  begin( sda_pin, scl_pin, c );
}

void MCR_AQM0802_SOFT_I2C::begin( uint16_t sda_pin, uint16_t scl_pin, uint8_t c ) {
  _sda = sda_pin;
  _scl = scl_pin;
  pinMode( _sda, INPUT );
  pinMode( _scl, INPUT );

  if( c <  1 ) c =  1;
  if( c > 63 ) c = 63;
  contrast = c;

  delay( 50 );
  writeCmd( 0x38 );     // Function set
  writeCmd( 0x39 );     // Function set
  writeCmd( 0x14 );     // Internal OSC frequency
  writeCmd( 0x70 | ( contrast & 0x0f) );      // Contrast set
  writeCmd( 0x54 | ( (contrast>>4) & 0x03) ); // Power/ICON/Contrast set
  writeCmd( 0x6c );     // Follower control
  delay( 200 );
  writeCmd( 0x38 );     // Function set
  writeCmd( 0x0c );     // Display On/OFF control
  writeCmd( 0x01);      // Clear Display
  delay( 15 );
}

/*
Ver.1.00 2024.07.11 作成(ArduinoライブラリのdigitalWrite,digitalReadを使ず、RAマイコンの入出力を使うプログラム)
Ver.2.00 2024.08.11 Arduinoライブラリの入出力を使用する
*/
